@@ -23,8 +23,9 @@ VER_MAJOR_MINOR = sys.version_info[:2]
 VER_FULL = sys.version_info[:3]
 ON_DARWIN = (platform.system() == 'Darwin')
 ON_WINDOWS = (platform.system() == 'Windows')
-ON_CONDA = True in [conda in pytest.__file__ for conda
+ON_CONDA = True in [conda in pytest.__file__.lower() for conda
                     in ['anaconda', 'miniconda']]
+ON_TRAVIS = 'TRAVIS' in os.environ and 'CI' in os.environ
 TEST_DIR = os.path.dirname(__file__)
 
 # pytest skip decorators
@@ -39,13 +40,15 @@ skip_if_on_unix = pytest.mark.skipif(not ON_WINDOWS, reason='Windows stuff')
 
 skip_if_on_darwin = pytest.mark.skipif(ON_DARWIN, reason='not Mac friendly')
 
+skip_if_on_travis = pytest.mark.skipif(ON_TRAVIS, reason='not Travis CI friendly')
+
 
 def sp(cmd):
     return subprocess.check_output(cmd, universal_newlines=True)
 
 
 class DummyStyler():
-    styles = defaultdict(None.__class__)
+    styles = defaultdict(str)
 
 
 class DummyBaseShell(BaseShell):
@@ -89,6 +92,7 @@ class DummyEnv(MutableMapping):
 
     DEFAULTS = {
         'XONSH_DEBUG': 1,
+        'XONSH_COLOR_STYLE': 'default',
     }
 
     def __init__(self, *args, **kwargs):
