@@ -1,5 +1,5 @@
 """Constructor for xonsh completer objects."""
-from collections import OrderedDict
+import collections
 
 from xonsh.completers.pip import complete_pip
 from xonsh.completers.man import complete_from_man
@@ -11,10 +11,12 @@ from xonsh.completers.python import (complete_python, complete_import,
                                      complete_python_mode)
 from xonsh.completers.commands import complete_skipper
 from xonsh.completers.completer import complete_completer
+from xonsh.completers.xompletions import complete_xonfig, complete_xontrib
+
 
 def default_completers():
-    """Creates a copy of the default completers.    """
-    return OrderedDict([
+    """Creates a copy of the default completers."""
+    return collections.OrderedDict([
         ('python_mode', complete_python_mode),
         ('base', complete_base),
         ('completer', complete_completer),
@@ -22,6 +24,8 @@ def default_completers():
         ('pip', complete_pip),
         ('cd', complete_cd),
         ('rmdir', complete_rmdir),
+        ('xonfig', complete_xonfig),
+        ('xontrib', complete_xontrib),
         ('bash', complete_from_bash),
         ('man', complete_from_man),
         ('import', complete_import),
