@@ -9,7 +9,6 @@ import textwrap
 import locale
 import builtins
 import warnings
-import traceback
 import contextlib
 import collections
 import collections.abc as cabc
@@ -18,10 +17,12 @@ from xonsh import __version__ as XONSH_VERSION
 from xonsh.lazyasd import LazyObject, lazyobject
 from xonsh.codecache import run_script_with_cache
 from xonsh.dirstack import _get_cwd
-from xonsh.foreign_shells import load_foreign_envs
+from xonsh.events import events
+from xonsh.foreign_shells import load_foreign_envs, load_foreign_aliases
+from xonsh.xontribs import update_context, prompt_xontrib_install
 from xonsh.platform import (
     BASH_COMPLETIONS_DEFAULT, DEFAULT_ENCODING, PATH_DEFAULT,
-    ON_WINDOWS, ON_LINUX
+    ON_WINDOWS, ON_LINUX, os_environ
 )
 
 from xonsh.tools import (
@@ -37,11 +38,30 @@ from xonsh.tools import (
     is_dynamic_cwd_width, to_dynamic_cwd_tuple, dynamic_cwd_tuple_to_str,
     is_logfile_opt, to_logfile_opt, logfile_opt_to_str, executables_in,
     is_nonstring_seq_of_strings, pathsep_to_upper_seq,
-    seq_to_upper_pathsep, print_color
+    seq_to_upper_pathsep, print_color, is_history_backend, to_itself,
+    swap_values,
 )
 import xonsh.prompt.base as prompt
 
 
+events.doc('on_envvar_new', """
+on_envvar_new(name: str, value: Any) -> None
+
+Fires after a new enviroment variable is created.
+Note: Setting envvars inside the handler might
+cause a recursion until the limit.
+""")
+
+
+events.doc('on_envvar_change', """
+on_envvar_change(name: str, oldvalue: Any, newvalue: Any) -> None
+
+Fires after an enviroment variable is changed.
+Note: Setting envvars inside the handler might
+cause a recursion until the limit.
+""")
+
+
 @lazyobject
 def HELP_TEMPLATE():
     return ('{{INTENSE_RED}}{envvar}{{NO_COLOR}}:\n\n'
@@ -111,9 +131,11 @@ def DEFAULT_ENSURERS():
                             to_completions_display_value, str),
     'COMPLETIONS_MENU_ROWS': (is_int, int, str),
     'COMPLETION_QUERY_LIMIT': (is_int, int, str),
+    'DIRSTACK_SIZE': (is_int, int, str),
     'DYNAMIC_CWD_WIDTH': (is_dynamic_cwd_width, to_dynamic_cwd_tuple,
                           dynamic_cwd_tuple_to_str),
     'DYNAMIC_CWD_ELISION_CHAR': (is_string, ensure_string, ensure_string),
+    'EXPAND_ENV_VARS': (is_bool, to_bool, bool_to_str),
     'FORCE_POSIX_PATHS': (is_bool, to_bool, bool_to_str),
     'FOREIGN_ALIASES_OVERRIDE': (is_bool, to_bool, bool_to_str),
     'FUZZY_PATH_COMPLETION': (is_bool, to_bool, bool_to_str),
@@ -138,12 +160,19 @@ def DEFAULT_ENSURERS():
                 seq_to_upper_pathsep),
     'PRETTY_PRINT_RESULTS': (is_bool, to_bool, bool_to_str),
     'PROMPT': (is_string_or_callable, ensure_string, ensure_string),
+    'PUSHD_MINUS': (is_bool, to_bool, bool_to_str),
+    'PUSHD_SILENT': (is_bool, to_bool, bool_to_str),
     'RAISE_SUBPROC_ERROR': (is_bool, to_bool, bool_to_str),
     'RIGHT_PROMPT': (is_string_or_callable, ensure_string, ensure_string),
     'BOTTOM_TOOLBAR': (is_string_or_callable, ensure_string, ensure_string),
     'SUBSEQUENCE_PATH_COMPLETION': (is_bool, to_bool, bool_to_str),
+    'SUGGEST_COMMANDS': (is_bool, to_bool, bool_to_str),
+    'SUGGEST_MAX_NUM': (is_int, int, str),
+    'SUGGEST_THRESHOLD': (is_int, int, str),
     'SUPPRESS_BRANCH_TIMEOUT_MESSAGE': (is_bool, to_bool, bool_to_str),
+    'UPDATE_COMPLETIONS_ON_KEYPRESS': (is_bool, to_bool, bool_to_str),
     'UPDATE_OS_ENVIRON': (is_bool, to_bool, bool_to_str),
+    'UPDATE_PROMPT_ON_KEYPRESS': (is_bool, to_bool, bool_to_str),
     'VC_BRANCH_TIMEOUT': (is_float, float, str),
     'VC_HG_SHOW_BRANCH': (is_bool, to_bool, bool_to_str),
     'VI_MODE': (is_bool, to_bool, bool_to_str),
@@ -157,7 +186,7 @@ def DEFAULT_ENSURERS():
     'XONSH_DEBUG': (always_false, to_debug, bool_or_int_to_str),
     'XONSH_ENCODING': (is_string, ensure_string, ensure_string),
     'XONSH_ENCODING_ERRORS': (is_string, ensure_string, ensure_string),
-    'XONSH_HISTORY_BACKEND': (is_string, ensure_string, ensure_string),
+    'XONSH_HISTORY_BACKEND': (is_history_backend, to_itself, ensure_string),
     'XONSH_HISTORY_FILE': (is_string, ensure_string, ensure_string),
     'XONSH_HISTORY_SIZE': (is_history_tuple, to_history_tuple, history_tuple_to_str),
     'XONSH_LOGIN': (is_bool, to_bool, bool_to_str),
@@ -169,7 +198,6 @@ def DEFAULT_ENSURERS():
     'XONSH_STORE_STDIN': (is_bool, to_bool, bool_to_str),
     'XONSH_TRACEBACK_LOGFILE': (is_logfile_opt, to_logfile_opt, logfile_opt_to_str),
     'XONSH_DATETIME_FORMAT': (is_string, ensure_string, ensure_string),
-    'UPDATE_PROMPT_ON_KEYPRESS': (is_bool, to_bool, bool_to_str),
     }
 
 
@@ -214,20 +242,19 @@ def xonshconfig(env):
     return xc
 
 
-def default_xonshrc():
+@default_value
+def default_xonshrc(env):
     """Creates a new instance of the default xonshrc tuple."""
     if ON_WINDOWS:
-        dxrc = (os.path.join(os.environ['ALLUSERSPROFILE'],
+        dxrc = (xonshconfig(env),
+                os.path.join(os_environ['ALLUSERSPROFILE'],
                              'xonsh', 'xonshrc'),
                 os.path.expanduser('~/.xonshrc'))
     else:
-        dxrc = ('/etc/xonshrc', os.path.expanduser('~/.xonshrc'))
+        dxrc = (xonshconfig(env), '/etc/xonshrc', os.path.expanduser('~/.xonshrc'))
     return dxrc
 
 
-DEFAULT_XONSHRC = LazyObject(default_xonshrc, globals(), 'DEFAULT_XONSHRC')
-
-
 # Default values should generally be immutable, that way if a user wants
 # to set them they have to do a copy and write them to the environment.
 # try to keep this sorted.
@@ -286,7 +313,9 @@ def DEFAULT_VALUES():
         'SUGGEST_MAX_NUM': 5,
         'SUGGEST_THRESHOLD': 3,
         'TITLE': DEFAULT_TITLE,
+        'UPDATE_COMPLETIONS_ON_KEYPRESS': False,
         'UPDATE_OS_ENVIRON': False,
+        'UPDATE_PROMPT_ON_KEYPRESS': False,
         'VC_BRANCH_TIMEOUT': 0.2 if ON_WINDOWS else 0.1,
         'VC_HG_SHOW_BRANCH': True,
         'VI_MODE': False,
@@ -295,7 +324,7 @@ def DEFAULT_VALUES():
         'XDG_DATA_HOME': os.path.expanduser(os.path.join('~', '.local',
                                                          'share')),
         'XONSHCONFIG': xonshconfig,
-        'XONSHRC': default_xonshrc(),
+        'XONSHRC': default_xonshrc,
         'XONSH_AUTOPAIR': False,
         'XONSH_CACHE_SCRIPTS': True,
         'XONSH_CACHE_EVERYTHING': False,
@@ -317,7 +346,6 @@ def DEFAULT_VALUES():
         'XONSH_STORE_STDOUT': False,
         'XONSH_TRACEBACK_LOGFILE': None,
         'XONSH_DATETIME_FORMAT': '%Y-%m-%d %H:%M',
-        'UPDATE_PROMPT_ON_KEYPRESS': False,
     }
     if hasattr(locale, 'LC_MESSAGES'):
         dv['LC_MESSAGES'] = locale.setlocale(locale.LC_MESSAGES)
@@ -405,7 +433,7 @@ def DEFAULT_DOCS():
         "- If ``$COMPLETIONS_DISPLAY`` is ``multi`` or ``true``, display completions\n"
         "  in multiple columns while typing.\n\n"
         'These option values are not case- or type-sensitive, so e.g.'
-        "writing ``$COMPLETIONS_DISPLAY = None``"
+        "writing ``$COMPLETIONS_DISPLAY = None`` "
         "and ``$COMPLETIONS_DISPLAY = 'none'`` are equivalent. Only usable with "
         "``$SHELL_TYPE=prompt_toolkit``"),
     'COMPLETIONS_CONFIRM': VarDocs(
@@ -572,10 +600,19 @@ def DEFAULT_DOCS():
         "in the same manner as ``$PROMPT``, see 'Customizing the Prompt' "
         'http://xon.sh/tutorial.html#customizing-the-prompt.',
         default='``xonsh.environ.DEFAULT_TITLE``'),
+    'UPDATE_COMPLETIONS_ON_KEYPRESS': VarDocs(
+        'Completions display is evaluated and presented whenever a key is '
+        'pressed. This avoids the need to press TAB, except to cycle through '
+        'the possibilities. This currently only affects the prompt-toolkit shell.'
+        ),
     'UPDATE_OS_ENVIRON': VarDocs(
-        "If True ``os.environ`` will always be updated "
+        "If True ``os_environ`` will always be updated "
         "when the xonsh environment changes. The environment can be reset to "
         "the default value by calling ``__xonsh_env__.undo_replace_env()``"),
+    'UPDATE_PROMPT_ON_KEYPRESS': VarDocs(
+        'Disables caching the prompt between commands, '
+        'so that it would be reevaluated on each keypress. '
+        'Disabled by default because of the incurred performance penalty.'),
     'VC_BRANCH_TIMEOUT': VarDocs(
         'The timeout (in seconds) for version control '
         'branch computations. This is a timeout per subprocess call, so the '
@@ -660,8 +697,10 @@ def DEFAULT_DOCS():
         '* ``XONSH_GITSTATUS_BEHIND``: ``↓·``\n'
     ),
     'XONSH_HISTORY_BACKEND': VarDocs(
-        'Set which history backend to use. Options are: ``json``, '
-        '``sqlite``, and ``dummy``. default is ``json``.'),
+        "Set which history backend to use. Options are: 'json', "
+        "'sqlite', and 'dummy'. The default is 'json'. "
+        '``XONSH_HISTORY_BACKEND`` also accepts a class type that inherits '
+        'from ``xonsh.history.base.History``, or its instance.'),
     'XONSH_HISTORY_FILE': VarDocs(
         'Location of history file (deprecated).',
         configurable=False, default="``~/.xonsh_history``"),
@@ -719,10 +758,6 @@ def DEFAULT_DOCS():
     'XONSH_DATETIME_FORMAT': VarDocs(
         'The format that is used for ``datetime.strptime()`` in various places'
         'i.e the history timestamp option'),
-    'UPDATE_PROMPT_ON_KEYPRESS': VarDocs(
-        'Disables caching the prompt between commands, '
-        'so that it would be reevaluated on each keypress. '
-        'Disabled by default because of the incurred performance penalty.'),
     }
 
 
@@ -747,16 +782,22 @@ class Env(cabc.MutableMapping):
     _arg_regex = None
 
     def __init__(self, *args, **kwargs):
-        """If no initial environment is given, os.environ is used."""
+        """If no initial environment is given, os_environ is used."""
         self._d = {}
+        # sentinel value for non existing envvars
+        self._no_value = object()
         self._orig_env = None
         self._ensurers = {k: Ensurer(*v) for k, v in DEFAULT_ENSURERS.items()}
         self._defaults = DEFAULT_VALUES
         self._docs = DEFAULT_DOCS
         if len(args) == 0 and len(kwargs) == 0:
-            args = (os.environ,)
+            args = (os_environ,)
         for key, val in dict(*args, **kwargs).items():
             self[key] = val
+        if ON_WINDOWS:
+            path_key = next((k for k in self._d if k.upper() == 'PATH'), None)
+            if path_key:
+                self['PATH'] = self._d.pop(path_key)
         if 'PATH' not in self._d:
             # this is here so the PATH is accessible to subprocs and so that
             # it can be modified in-place in the xonshrc file
@@ -789,21 +830,21 @@ class Env(cabc.MutableMapping):
         return ctx
 
     def replace_env(self):
-        """Replaces the contents of os.environ with a detyped version
+        """Replaces the contents of os_environ with a detyped version
         of the xonsh environement.
         """
         if self._orig_env is None:
-            self._orig_env = dict(os.environ)
-        os.environ.clear()
-        os.environ.update(self.detype())
+            self._orig_env = dict(os_environ)
+        os_environ.clear()
+        os_environ.update(self.detype())
 
     def undo_replace_env(self):
-        """Replaces the contents of os.environ with a detyped version
+        """Replaces the contents of os_environ with a detyped version
         of the xonsh environement.
         """
         if self._orig_env is not None:
-            os.environ.clear()
-            os.environ.update(self._orig_env)
+            os_environ.clear()
+            os_environ.update(self._orig_env)
             self._orig_env = None
 
     def get_ensurer(self, key,
@@ -913,6 +954,8 @@ class Env(cabc.MutableMapping):
         ensurer = self.get_ensurer(key)
         if not ensurer.validate(val):
             val = ensurer.convert(val)
+        # existing envvars can have any value including None
+        old_value = self._d[key] if key in self._d else self._no_value
         self._d[key] = val
         if self.detypeable(val):
             self._detyped = None
@@ -920,14 +963,20 @@ class Env(cabc.MutableMapping):
                 if self._orig_env is None:
                     self.replace_env()
                 else:
-                    os.environ[key] = ensurer.detype(val)
+                    os_environ[key] = ensurer.detype(val)
+        if old_value is self._no_value:
+            events.on_envvar_new.fire(name=key, value=val)
+        elif old_value != val:
+            events.on_envvar_change.fire(name=key,
+                                         oldvalue=old_value,
+                                         newvalue=val)
 
     def __delitem__(self, key):
         val = self._d.pop(key)
         if self.detypeable(val):
             self._detyped = None
-            if self.get('UPDATE_OS_ENVIRON') and key in os.environ:
-                del os.environ[key]
+            if self.get('UPDATE_OS_ENVIRON') and key in os_environ:
+                del os_environ[key]
 
     def get(self, key, default=None):
         """The environment will look up default values from its own defaults if a
@@ -1009,10 +1058,10 @@ def load_static_config(ctx, config=None):
         config = env['XONSHCONFIG'] = xonshconfig(env)
     if os.path.isfile(config):
         # Note that an Env instance at __xonsh_env__ has not been started yet,
-        # per se, so we have to use os.environ
-        encoding = os.environ.get('XONSH_ENCODING',
+        # per se, so we have to use os_environ
+        encoding = os_environ.get('XONSH_ENCODING',
                                   DEFAULT_VALUES.get('XONSH_ENCODING', 'utf8'))
-        errors = os.environ.get('XONSH_ENCODING_ERRORS',
+        errors = os_environ.get('XONSH_ENCODING_ERRORS',
                                 DEFAULT_VALUES.get('XONSH_ENCODING_ERRORS',
                                                    'surrogateescape'))
         with open(config, 'r', encoding=encoding, errors=errors) as f:
@@ -1039,36 +1088,26 @@ def load_static_config(ctx, config=None):
     return conf
 
 
-def xonshrc_context(rcfiles=None, execer=None, initial=None):
-    """Attempts to read in xonshrc file, and return the contents."""
-    loaded = builtins.__xonsh_env__['LOADED_RC_FILES'] = []
-    if initial is None:
-        env = {}
-    else:
-        env = initial
-    if rcfiles is None or execer is None:
+def xonshrc_context(rcfiles=None, execer=None, ctx=None, env=None, login=True):
+    """Attempts to read in all xonshrc files and return the context."""
+    loaded = env['LOADED_RC_FILES'] = []
+    ctx = {} if ctx is None else ctx
+    if rcfiles is None:
         return env
     env['XONSHRC'] = tuple(rcfiles)
     for rcfile in rcfiles:
         if not os.path.isfile(rcfile):
             loaded.append(False)
             continue
-        try:
-            run_script_with_cache(rcfile, execer, env)
-            loaded.append(True)
-        except SyntaxError as err:
-            loaded.append(False)
-            exc = traceback.format_exc()
-            msg = '{0}\nsyntax error in xonsh run control file {1!r}: {2!s}'
-            warnings.warn(msg.format(exc, rcfile, err), RuntimeWarning)
-            continue
-        except Exception as err:
-            loaded.append(False)
-            exc = traceback.format_exc()
-            msg = '{0}\nerror running xonsh run control file {1!r}: {2!s}'
-            warnings.warn(msg.format(exc, rcfile, err), RuntimeWarning)
-            continue
-    return env
+        _, ext = os.path.splitext(rcfile)
+        if ext == '.json':
+            status = static_config_run_control(rcfile, ctx, env, execer=execer,
+                                               login=login)
+        else:
+            status = xonsh_script_run_control(rcfile, ctx, env, execer=execer,
+                                              login=login)
+        loaded.append(status)
+    return ctx
 
 
 def windows_foreign_env_fixes(ctx):
@@ -1081,8 +1120,8 @@ def windows_foreign_env_fixes(ctx):
     # /c/Windows/System32 syntax instead of C:\\Windows\\System32
     # which messes up these environment variables for xonsh.
     for ev in ['PATH', 'TEMP', 'TMP']:
-        if ev in os.environ:
-            ctx[ev] = os.environ[ev]
+        if ev in os_environ:
+            ctx[ev] = os_environ[ev]
         elif ev in ctx:
             del ctx[ev]
     ctx['PWD'] = _get_cwd() or ''
@@ -1094,28 +1133,72 @@ def foreign_env_fixes(ctx):
         del ctx['PROMPT']
 
 
-def default_env(env=None, config=None, login=True):
+def static_config_run_control(filename, ctx, env, execer=None, login=True):
+    """Loads a static config file and applies it as a run control."""
+    if not login:
+        return
+    conf = load_static_config(env, config=filename)
+    # load foreign shells
+    foreign_env = load_foreign_envs(shells=conf.get('foreign_shells', ()),
+                                    issue_warning=False)
+    if ON_WINDOWS:
+        windows_foreign_env_fixes(foreign_env)
+    foreign_env_fixes(foreign_env)
+    env.update(foreign_env)
+    aliases = builtins.aliases
+    foreign_aliases = load_foreign_aliases(config=filename, issue_warning=True)
+    for k, v in foreign_aliases.items():
+        if k in aliases:
+            msg = ('Skipping application of {0!r} alias from foreign shell '
+                   '(loaded from {1!r}) since it shares a name with an '
+                   'existing xonsh alias.')
+            print(msg.format(k, filename))
+        else:
+            aliases[k] = v
+    # load xontribs
+    names = conf.get('xontribs', ())
+    for name in names:
+        update_context(name, ctx=ctx)
+    if getattr(update_context, 'bad_imports', None):
+        prompt_xontrib_install(update_context.bad_imports)
+        del update_context.bad_imports
+    # Do static config environment last, to allow user to override any of
+    # our environment choices
+    env.update(conf.get('env', ()))
+    return True
+
+
+def xonsh_script_run_control(filename, ctx, env, execer=None, login=True):
+    """Loads a xonsh file and applies it as a run control."""
+    if execer is None:
+        return False
+    updates = {'__file__': filename, '__name__': os.path.abspath(filename)}
+    try:
+        with swap_values(ctx, updates):
+            run_script_with_cache(filename, execer, ctx)
+        loaded = True
+    except SyntaxError as err:
+        msg = 'syntax error in xonsh run control file {0!r}: {1!s}'
+        print_exception(msg.format(filename, err))
+        loaded = False
+    except Exception as err:
+        msg = 'error running xonsh run control file {0!r}: {1!s}'
+        print_exception(msg.format(filename, err))
+        loaded = False
+    return loaded
+
+
+def default_env(env=None):
     """Constructs a default xonsh environment."""
     # in order of increasing precedence
     ctx = dict(BASE_ENV)
-    ctx.update(os.environ)
+    ctx.update(os_environ)
     ctx['PWD'] = _get_cwd() or ''
     # other shells' PROMPT definitions generally don't work in XONSH:
     try:
         del ctx['PROMPT']
     except KeyError:
         pass
-    if login:
-        conf = load_static_config(ctx, config=config)
-        foreign_env = load_foreign_envs(shells=conf.get('foreign_shells', ()),
-                                        issue_warning=False)
-        if ON_WINDOWS:
-            windows_foreign_env_fixes(foreign_env)
-        foreign_env_fixes(foreign_env)
-        ctx.update(foreign_env)
-        # Do static config environment last, to allow user to override any of
-        # our environment choices
-        ctx.update(conf.get('env', ()))
     # finalize env
     if env is not None:
         ctx.update(env)
